#include <SDL.h>

// ^ This dummy autoincrementing timer provides suitable testing with valgrind.
static struct{ int operator()(...){return 0;} operator int() const { return 0; } }
    C80,_dos_ds,  _farsetsel;
static SDL_Surface* surface = 0;
static unsigned char vmem[64000];
static unsigned palette[256] = { 0 }, palettecounter=0;
static void _farnspokeb(unsigned o, unsigned pix)
{
    vmem[ (o&0xFFFF) % 64000 ] = pix;
}
static void outportb(unsigned o, unsigned value)
{
    if(o != 0x3C9) return;
    value = (value << 2) | (value >> 4);
    unsigned& v = palette[palettecounter/3];
    unsigned shift = 16 - (palettecounter%3)*8;
    v &= ~ (0xFF << shift);
    v |=  (value << shift);
    palettecounter = (palettecounter+1) % 768;
}

bool DisplayTrueColor = false, GrabWindow = true;

static void textmode(int mode)
{
    if(mode == 0x13)
    {
        // set graphics
        surface = SDL_SetVideoMode(320, DisplayTrueColor ? 400 : 200, 32, 0);
        SDL_EnableKeyRepeat(150, 30);
        if(GrabWindow)
        {
            SDL_WM_GrabInput(SDL_GRAB_ON);
            SDL_ShowCursor(SDL_DISABLE);
        }
    }
    else
    {
        // unset graphics
        SDL_Quit();
    }
}
static void updatescreen(unsigned* buf)
{
    SDL_LockSurface(surface);

#if !TRUECOLOR_ONLY
    if(DisplayTrueColor)
#endif
        memcpy( surface->pixels, buf, 320*200*4 );

#if !TRUECOLOR_ONLY
    unsigned* target = 64000*DisplayTrueColor + (unsigned*)surface->pixels;
    for(unsigned p=0; p<64000; ++p)
        target[p] = palette[vmem[p]];
#endif

    #if REPLAY
        static FILE* fp = popen("lzop -7 > frames.lzop", "w");
        fwrite(target, 64000, 4, fp);
        fflush(fp);
    #endif

    SDL_UnlockSurface(surface);
    /*static int flip=0;
    if(flip++ % 2  == 0)*/
    SDL_Flip(surface);
}
static SDL_Event pending_ev;
static int kbhit()
{
    if(pending_ev.type == SDL_KEYDOWN) return 1;
    if(!SDL_PollEvent(&pending_ev)) return 0;
    if(pending_ev.type == SDL_QUIT) SDL_Quit();
    if(pending_ev.type != SDL_KEYDOWN)
    {
        pending_ev.type = 0;
        return kbhit();
    }
    return 1;
}
static int getch()
{
    if(pending_ev.type == SDL_KEYDOWN)
        { pending_ev.type = 0; return pending_ev.key.keysym.sym; }
    SDL_Event ev;
    SDL_WaitEvent(&ev);
    if(ev.type == SDL_KEYDOWN) return ev.key.keysym.sym;
    return getch();
}
